/*
* ECMD-commands to handle Z80 interfacing
*
* Copyright (c) 2011 by Sprite_tm <jeroen@spritesmods.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 3
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* For more information on the GPL, please go to:
* http://www.gnu.org/copyleft/gpl.html
*/

#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "config.h"
#include "core/debug.h"
#include "core/bit-macros.h"

#include "protocols/ecmd/ecmd-base.h"
#include "z80interface.h"

static char *findNextArg(char *str) {
	while (*str!=' ' && *str!=0) str++;
	while (*str==' ') str++;
	return str;
}


int16_t parse_cmd_z80reset(char *cmd, char *output, uint16_t len) {
	z80Reset();
	return ECMD_FINAL(0);
}

int16_t parse_cmd_z80read(char *cmd, char *output, uint16_t len) {
	unsigned long addr;
	unsigned char data;
	cmd=findNextArg(cmd);
	addr=strtol(cmd, NULL, 16);
	z80TakeOverBus();
	data=z80ReadMem(addr);
	z80ReleaseBus();
	sprintf(output, "%02X", data);
	return ECMD_FINAL(2);
}

int16_t parse_cmd_z80write(char *cmd, char *output, uint16_t len) {
	unsigned long addr;
	unsigned char data;
	cmd=findNextArg(cmd);
	addr=strtol(cmd, NULL, 16);
	cmd=findNextArg(cmd);
	data=strtol(cmd, NULL, 16);
	z80TakeOverBus();
	z80SetMem(addr, data);
	z80ReleaseBus();
	return ECMD_FINAL(0);
}

void z80init() {
	z80IoInit();
}

/*
  -- Ethersex META --
  init(z80init)
  ecmd_feature(z80read, "z80read", ADDR, Get byte at the z80 memory address ADDR)
  ecmd_feature(z80write, "z80write", ADDR DATA, Write byte DATA to the z80 memory address ADDR)
  ecmd_feature(z80reset, "z80reset", , Reset Z80)
*/

