/*
* Z80 interfacing
*
* Copyright (c) 2011 by Sprite_tm <jeroen@spritesmods.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 3
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* For more information on the GPL, please go to:
* http://www.gnu.org/copyleft/gpl.html
*/
#include "z80interface.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define nop()  __asm__ __volatile__("nop")


//Unused.
ISR(INT1_vect) {
}

//Sends a 16-bit address word to the 2 latches. Needs the bus taken over.
static void z80SetAddr(unsigned int addr) {
	unsigned char byte=0;
	
	//First the latch for the lower byte...
	DDRA=0xFF;
	if (addr&(1<<Z80_L1_D0)) byte|=(1<<0);
	if (addr&(1<<Z80_L1_D1)) byte|=(1<<1);
	if (addr&(1<<Z80_L1_D2)) byte|=(1<<2);
	if (addr&(1<<Z80_L1_D3)) byte|=(1<<3);
	if (addr&(1<<Z80_L1_D4)) byte|=(1<<4);
	if (addr&(1<<Z80_L1_D5)) byte|=(1<<5);
	if (addr&(1<<Z80_L1_D6)) byte|=(1<<6);
	if (addr&(1<<Z80_L1_D7)) byte|=(1<<7);
	PORTA=byte;
	nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC|=(1<<Z80_L1_LE);
	nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC&=~(1<<Z80_L1_LE);

	//then the upper byte.
	byte=0;
	if (addr&(1<<Z80_L2_D0)) byte|=(1<<0);
	if (addr&(1<<Z80_L2_D1)) byte|=(1<<1);
	if (addr&(1<<Z80_L2_D2)) byte|=(1<<2);
	if (addr&(1<<Z80_L2_D3)) byte|=(1<<3);
	if (addr&(1<<Z80_L2_D4)) byte|=(1<<4);
	if (addr&(1<<Z80_L2_D5)) byte|=(1<<5);
	if (addr&(1<<Z80_L2_D6)) byte|=(1<<6);
	if (addr&(1<<Z80_L2_D7)) byte|=(1<<7);
	PORTA=byte;
	nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTB|=(1<<Z80_L2_LE);
	nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTB&=~(1<<Z80_L2_LE);
	nop(); nop(); nop(); nop(); nop(); nop(); 
}

//Send a byte to the Z80s data bus.
static void z80SetData(unsigned char data) {
	unsigned char byte=0;
	if (data&(1<<Z80_D_D0)) byte|=(1<<0);
	if (data&(1<<Z80_D_D1)) byte|=(1<<1);
	if (data&(1<<Z80_D_D2)) byte|=(1<<2);
	if (data&(1<<Z80_D_D3)) byte|=(1<<3);
	if (data&(1<<Z80_D_D4)) byte|=(1<<4);
	if (data&(1<<Z80_D_D5)) byte|=(1<<5);
	if (data&(1<<Z80_D_D6)) byte|=(1<<6);
	if (data&(1<<Z80_D_D7)) byte|=(1<<7);
	PORTA=byte;
}

//Get a byte from the Z80s data bus.
static unsigned char z80GetData() {
	unsigned char byte=0;
	if (PINA&(1<<0)) byte|=(1<<Z80_D_D0);
	if (PINA&(1<<1)) byte|=(1<<Z80_D_D1);
	if (PINA&(1<<2)) byte|=(1<<Z80_D_D2);
	if (PINA&(1<<3)) byte|=(1<<Z80_D_D3);
	if (PINA&(1<<4)) byte|=(1<<Z80_D_D4);
	if (PINA&(1<<5)) byte|=(1<<Z80_D_D5);
	if (PINA&(1<<6)) byte|=(1<<Z80_D_D6);
	if (PINA&(1<<7)) byte|=(1<<Z80_D_D7);
	return byte;
}

//Init io
void z80IoInit() {
	PORTB|=(1<<Z80_L_OE)|(1<<Z80_L1_LE);
	PORTC|=(1<<Z80_L2_LE);
	PORTD|=(1<<Z80_BREQOUT);
	PORTC|=(1<<Z80_RESET);
	DDRA=0;
	DDRC|=(1<<Z80_L1_LE);
	DDRB|=(1<<Z80_L2_LE)|(1<<Z80_L_OE);
	DDRD|=(1<<Z80_BREQOUT);
//	EICRA|=(1<<2); //int on any edge
//	EIMSK|=(1<<1); //int 1 ena
}

//Undocumented function: Reset the Z80
void z80Reset() {
	DDRC|=(1<<Z80_RESET);
	PORTC&=~(1<<Z80_RESET);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	PORTC|=(1<<Z80_RESET);
	DDRC&=~(1<<Z80_RESET);
}
//Take over the bus so we can do reads and writes.
void z80TakeOverBus() {
	if (!(PORTD&(1<<Z80_BREQOUT))) return;
	//Wait for GPU to access hardware
	while (PIND&(1<<Z80_BREQIN)) ;
	//Claim bus after it
	PORTD&=~(1<<Z80_BREQOUT);
	//Wait till GPU is done.
	while (!(PIND&(1<<Z80_BREQIN))) ;
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	//Get ready to take over bus
	PORTC|=(1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	//We control the bus now.
	DDRC|=(1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR);
	DDRA=0xff;
	PORTB&=~(1<<Z80_L_OE);
}

//Release the bus so the Z80 can go on with the program.
void z80ReleaseBus() {
	DDRC&=~((1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR));
	DDRA=0;
	PORTC&=~((1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR));
	PORTB|=(1<<Z80_L_OE);
	PORTD|=(1<<Z80_BREQOUT);
}

//Reads a byte from the Z80s memory. Call z80TakeOverBus() before using!
unsigned char z80ReadMem(unsigned int addr) {
	unsigned char ret;
	z80SetAddr(addr);
	DDRA=0;
	PORTC&=~(1<<Z80_RD);
	PORTC&=~(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	ret=z80GetData();
	PORTC|=(1<<Z80_RD);
	PORTC|=(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	return ret;
}

//Write a byte into the Z80s memory. Call z80TakeOverBus() before using!
void z80SetMem(unsigned int addr, unsigned char data) {
	z80SetAddr(addr);
	z80SetData(data);
	PORTC&=~(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC&=~(1<<Z80_WR);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC|=(1<<Z80_WR);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC|=(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	return;
}
