#include "io.h"
#include <avr/io.h>
#include <avr/delay.h>

#define nop()  __asm__ __volatile__("nop")


static void z80SetAddr(unsigned int addr) {
	unsigned char byte=0;
	DDRA=0xFF;
	if (addr&(1<<Z80_L1_D0)) byte|=(1<<0);
	if (addr&(1<<Z80_L1_D1)) byte|=(1<<1);
	if (addr&(1<<Z80_L1_D2)) byte|=(1<<2);
	if (addr&(1<<Z80_L1_D3)) byte|=(1<<3);
	if (addr&(1<<Z80_L1_D4)) byte|=(1<<4);
	if (addr&(1<<Z80_L1_D5)) byte|=(1<<5);
	if (addr&(1<<Z80_L1_D6)) byte|=(1<<6);
	if (addr&(1<<Z80_L1_D7)) byte|=(1<<7);
	PORTA=byte;
	nop(); nop(); 
	PORTC|=(1<<Z80_L1_LE);
	nop(); nop();
	PORTC&=~(1<<Z80_L1_LE);

	byte=0;
	if (addr&(1<<Z80_L2_D0)) byte|=(1<<0);
	if (addr&(1<<Z80_L2_D1)) byte|=(1<<1);
	if (addr&(1<<Z80_L2_D2)) byte|=(1<<2);
	if (addr&(1<<Z80_L2_D3)) byte|=(1<<3);
	if (addr&(1<<Z80_L2_D4)) byte|=(1<<4);
	if (addr&(1<<Z80_L2_D5)) byte|=(1<<5);
	if (addr&(1<<Z80_L2_D6)) byte|=(1<<6);
	if (addr&(1<<Z80_L2_D7)) byte|=(1<<7);
	PORTA=byte;
	nop(); nop(); 
	PORTB|=(1<<Z80_L2_LE);
	nop(); nop();
	PORTB&=~(1<<Z80_L2_LE);
}

static void z80SetData(unsigned char data) {
	unsigned char byte=0;
	if (data&(1<<Z80_D_D0)) byte|=(1<<0);
	if (data&(1<<Z80_D_D1)) byte|=(1<<1);
	if (data&(1<<Z80_D_D2)) byte|=(1<<2);
	if (data&(1<<Z80_D_D3)) byte|=(1<<3);
	if (data&(1<<Z80_D_D4)) byte|=(1<<4);
	if (data&(1<<Z80_D_D5)) byte|=(1<<5);
	if (data&(1<<Z80_D_D6)) byte|=(1<<6);
	if (data&(1<<Z80_D_D7)) byte|=(1<<7);
	PORTA=byte;
}

static unsigned char z80GetData() {
	unsigned char byte=0;
	if (PINA&(1<<0)) byte|=(1<<Z80_D_D0);
	if (PINA&(1<<1)) byte|=(1<<Z80_D_D1);
	if (PINA&(1<<2)) byte|=(1<<Z80_D_D2);
	if (PINA&(1<<3)) byte|=(1<<Z80_D_D3);
	if (PINA&(1<<4)) byte|=(1<<Z80_D_D4);
	if (PINA&(1<<5)) byte|=(1<<Z80_D_D5);
	if (PINA&(1<<6)) byte|=(1<<Z80_D_D6);
	if (PINA&(1<<7)) byte|=(1<<Z80_D_D7);
	return byte;
}

void ioInit() {
	PORTB=(1<<Z80_L_OE);
	PORTD=(1<<Z80_BREQOUT);
	PORTC=(1<<Z80_RESET);
	DDRA=0;
	DDRC=(1<<Z80_L1_LE);
	DDRB=(1<<Z80_L2_LE)|(1<<Z80_L_OE);
	DDRD=(1<<Z80_BREQOUT);
}

void z80Reset() {
	DDRC|=(1<<Z80_RESET);
	PORTC&=~(1<<Z80_RESET);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	PORTC|=(1<<Z80_RESET);
	DDRC&=~(1<<Z80_RESET);
}

void z80TakeOverBus() {
	PORTD&=~(1<<Z80_BREQOUT);
	//Get ready to take over bus
	PORTC|=(1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	//We control the bus now.
	DDRC|=(1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR);
	DDRA=0xff;
	PORTB&=~(1<<Z80_L_OE);
}

void z80ReleaseBus() {
	DDRC&=~((1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR));
	DDRA=0;
	PORTC&=~((1<<Z80_MREQ)|(1<<Z80_IORQ)|(1<<Z80_RD)|(1<<Z80_WR));
	PORTB|=(1<<Z80_L_OE);
	PORTD|=(1<<Z80_BREQOUT);
}

unsigned char z80ReadMem(unsigned int addr) {
	unsigned char ret;
	z80SetAddr(addr);
	DDRA=0;
	PORTC&=~(1<<Z80_RD);
	PORTC&=~(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	ret=z80GetData();
	PORTC|=(1<<Z80_RD);
	PORTC|=(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	return ret;
}

void z80SetMem(unsigned int addr, unsigned char data) {
	z80SetAddr(addr);
	PORTC&=~(1<<Z80_MREQ);
	z80SetData(data);
	PORTC&=~(1<<Z80_WR);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	PORTC|=(1<<Z80_WR);
	PORTC|=(1<<Z80_MREQ);
	nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop(); 
	return;
}
